%SLR class definition
%
%   Usage:
%      slr=slr();

classdef slr
	properties (SetAccess=public) 
		deltathickness = NaN;
		maxiter        = 0;
		reltol         = 0;
		abstol         = 0;
		love_h         = 0; %provided by PREM model
		love_k         = 0; %ideam
		rigid         = 0;
		elastic         = 0;
		eustatic         = 0;
	end
	methods
		function self = slr(varargin) % {{{
			switch nargin
				case 0
					self=setdefaultparameters(self);
				otherwise
					error('constructor not supported');
			end
		end % }}}
		function self = setdefaultparameters(self) % {{{
		
		%Convergence criterion: absolute, relative and residual
		self.reltol=0.01; %1 percent
		self.abstol=0.001; %1 mm of sea level rise

		%maximum of non-linear iterations.
		self.maxiter=5;

		%computational flags: 
		self.rigid=1;
		self.elastic=1;
		self.eustatic=1;
		
		end % }}}
		function md = checkconsistency(self,md,solution,analyses) % {{{

			if ~ismember(SealevelriseAnalysisEnum(),analyses), return; end
			md = checkfield(md,'fieldname','slr.deltathickness','NaN',1,'Inf',1,'size',[md.mesh.numberofvertices 1]);
			md = checkfield(md,'fieldname','slr.love_h','NaN',1,'Inf',1);
			md = checkfield(md,'fieldname','slr.love_k','NaN',1,'Inf',1);
			md = checkfield(md,'fieldname','slr.reltol','size',[1 1]);
			md = checkfield(md,'fieldname','slr.abstol','size',[1 1]);
			md = checkfield(md,'fieldname','slr.maxiter','size',[1 1],'>=',1);

			%check that love numbers are provided at the same level of accuracy: 
			if (size(self.love_h,1) ~= size(self.love_k,1)),
				error('slr error message: love numbers should be provided at the same level of accuracy');
			end

		end % }}}
		function disp(self) % {{{
			disp(sprintf('   slr parameters:'));

			fielddisplay(self,'deltathickness','thickness change (main loading of the slr solution core [m]');
			fielddisplay(self,'reltol','sea level rise relative convergence criterion, NaN: not applied');
			fielddisplay(self,'abstol','sea level rise absolute convergence criterion, NaN: not applied');
			fielddisplay(self,'maxiter','maximum number of nonlinear iterations');
			fielddisplay(self,'love_h','love load number for radial displacement');
			fielddisplay(self,'love_k','love load number for gravitational potential perturbation');
			fielddisplay(self,'rigid','rigid earth graviational potential perturbation');
			fielddisplay(self,'elastic','elastic earth graviational potential perturbation');
			fielddisplay(self,'eustatic','eustatic sea level rise');

		end % }}}
		function marshall(self,md,fid) % {{{
			WriteData(fid,'object',self,'class','sealevelrise','fieldname','deltathickness','format','DoubleMat','mattype',1);
			WriteData(fid,'object',self,'class','sealevelrise','fieldname','reltol','format','Double');
			WriteData(fid,'object',self,'class','sealevelrise','fieldname','abstol','format','Double');
			WriteData(fid,'object',self,'class','sealevelrise','fieldname','maxiter','format','Integer');
			WriteData(fid,'object',self,'class','sealevelrise','fieldname','love_h','format','DoubleMat','mattype',1);
			WriteData(fid,'object',self,'class','sealevelrise','fieldname','love_k','format','DoubleMat','mattype',1);
			WriteData(fid,'object',self,'class','sealevelrise','fieldname','rigid','format','Boolean');
			WriteData(fid,'object',self,'class','sealevelrise','fieldname','elastic','format','Boolean');
			WriteData(fid,'object',self,'class','sealevelrise','fieldname','eustatic','format','Boolean');
		end % }}}
		function savemodeljs(self,fid,modelname) % {{{
		
			writejs1Darray(fid,[modelname '.srl.deltathickness'],self.deltathickness);
			writejsdouble(fid,[modelname '.slr.reltol'],self.reltol);
			writejsdouble(fid,[modelname '.slr.abstol'],self.abstol);
			writejsdouble(fid,[modelname '.slr.maxiter'],self.maxiter);
			writejs1Darray(fid,[modelname '.srl.love_h'],self.love_h);
			writejs1Darray(fid,[modelname '.srl.love_k'],self.love_k);
			writejsdouble(fid,[modelname '.slr.rigid'],self.rigid);
			writejsdouble(fid,[modelname '.slr.eustatic'],self.eustatic);

		end % }}}
	end
end
